﻿/***************************************************************
 * Copyright: xiaotie@gmail.com(http://xiaotie.cnblogs.com)
 * *************************************************************/
using System;
using System.Collections.Generic;
using System.Reflection;
using System.Globalization;
using Microsoft.CSharp;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace Orc.Shell.Core
{
    public class CmdDispatcher
    {
        public static readonly String[] SPLITS = { " ", "\t", "\n" };

        public Context Context { get; private set; }
        public String InputCmdString { get; private set; }
        public CmdDispatcher(Context cxt, String cmd)
        {
            Context = cxt;
            InputCmdString = cmd;
        }

        public static Assembly Find(String filePath)
        {
            return AppDomain.CurrentDomain.Load(filePath);
        }

        public void Dispatch()
        {
            String[] results = InputCmdString.Split(SPLITS, StringSplitOptions.None);
            if(results.Length == 0) return;

            String cmd = results[0];
            String mark = String.Empty;
            IList<String> args = new List<String>();

            Int32 argIndex = 1;

            if (results.Length > 1 && results[1].StartsWith("-"))
            {
                argIndex ++;
                mark = results[1];
            }

            for(;argIndex < results.Length;argIndex++)
            {
                args.Add(results[argIndex]);
            }

            switch (cmd.ToLower())
                {
                    case "debug":   // 开启debug开关
                        Context.Debug = true;
                        break;
                    case "undebug": // 关闭debug开关
                        Context.Debug = false;
                        break;
                    case "cdc":     // 改变命名空间
                        new CdClassCmdHandler(Context, InputCmdString, mark, args).Run();
                        break;
                    case "lsc":     // 列出命名空间的内容
                    case "dirc":
                        new ListClassCmdHandler(Context, InputCmdString, mark, args).Run();
                        break;
                    case "my":      // 列出用户变量
                        new ListInstanceCmdHandler(Context, InputCmdString, mark, args).Run();
                        break;
                    case "alias":   // 列出alias列表
                        new ListAliasCmdHandler(Context, InputCmdString, mark, args).Run();
                        break;
                    default:
                        String fullCmd = Context.GetFullCmd(cmd);
                        if (fullCmd != null)    // 处理 alias
                        {
                            if (mark != null) fullCmd += " " + mark;
                            if (args != null && args.Count > 0)
                            {
                                foreach(String s in args)
                                {
                                    fullCmd += " " + s;
                                }
                            }

                            Context.Invoke(fullCmd);
                        }
                        else                   // 编译代码并运行
                        {
                            new CscCmdHandler(Context, InputCmdString).Run();
                        }
                        break;
                }

            return;
        }
    }

    public abstract class CmdHandler
    {
        public Context Context { get; private set; }
        public String InputCmdString { get; private set; }
        public String Mark { get; private set; }
        public IList<String> Args { get; private set; }
        public String OutputCmdString { get; set; }

        public CmdHandler(Context cxt, String fullCmdString, String mark, IList<String> args)
        {
            Context = cxt;
            InputCmdString = fullCmdString;
            Mark = mark;
            Args = args;
        }

        public void ListClass(String match)
        {
            Console.WriteLine(Context.TypeManager.Now.FullName);
            Console.WriteLine("--------------------------------------");
            Context.TypeManager.Now.ListDir(match);
            Context.TypeManager.Now.ListType(match);
        }

        public void ShowLocation()
        {
            Console.WriteLine("Location: " + Context.TypeManager.Now.FullName); 
        }

        public abstract void Run();
    }

    public class ListClassCmdHandler : CmdHandler
    {
        public ListClassCmdHandler(Context cxt, String fullCmdString, String mark, IList<String> args)
            : base(cxt, fullCmdString, mark, args)
        {
        }

        public override void Run()
        {
            String match = null;

            if (Args != null && Args.Count == 1)
            {
                match = Args[0].Trim();
            }

            ListClass(match);
        }
    }

    public class CdClassCmdHandler : CmdHandler
    {
        public CdClassCmdHandler(Context cxt, String fullCmdString, String mark, IList<String> args)
            : base(cxt, fullCmdString, mark, args)
        {
        }

        public override void Run()
        {
            String match = null;

            if (Args != null && Args.Count == 1)
            {
                match = Args[0].Trim();
            }

            if (String.IsNullOrEmpty(match) || match.Equals("."))
            {
                ShowLocation();
                return;
            }

            if (match.Equals(".."))
            {
                Context.TypeManager.StepUp();
                ShowLocation();
            }
            else
            {
                TypeDictionary matchDic = null;

                IList<TypeDictionary> matchList = new List<TypeDictionary>();
                foreach (TypeDictionary td in Context.TypeManager.Now.SubTypeDictionary.Values)
                {
                    if (td.Name.Equals(match))
                    {
                        matchDic = td;
                        Context.TypeManager.StepDown(match);
                        ShowLocation();
                        return;
                    }

                    if (td.Name.StartsWith(match))
                    {
                        matchList.Add(td);
                    }
                }

                if (matchList.Count == 0)
                {
                    Console.WriteLine("指定的命名空间不存在.");
                }
                else if (matchList.Count == 1)  // 当只有1个匹配项时，则进入该匹配项
                {
                    Context.TypeManager.StepDown(matchList[0].Name);
                    ShowLocation();
                    return;
                }
                else // 超过1个匹配项时，保留在当前命名空间，同时打印全部匹配项
                {
                    foreach (TypeDictionary td in matchList)
                    {
                        Console.WriteLine(Context.EnsureAtLeastLength(td.Name,24));
                    }
                }
            }
        }
    }

    public class ListInstanceCmdHandler : CmdHandler
    {
        public ListInstanceCmdHandler(Context cxt, String fullCmdString, String mark, IList<String> args)
            : base(cxt, fullCmdString, mark, args)
        {
        }

        public override void Run()
        {
            Context.ListInstances();
        }
    }

    public class ListAliasCmdHandler : CmdHandler
    {
        public ListAliasCmdHandler(Context cxt, String fullCmdString, String mark, IList<String> args)
            : base(cxt, fullCmdString, mark, args)
        {
        }

        public override void Run()
        {
            Context.ListAlias();
        }

    }

    public class CscCmdHandler : CmdHandler
    {
        public CscCmdHandler(Context cxt, String code)
            : base(cxt, code, String.Empty,null)
        {
        }

        public override void Run()
        {
            if (String.IsNullOrEmpty(InputCmdString)) return;
            String fullCmd = String.Empty;
            if (Context.TypeManager.Now != Context.TypeManager.Root)
            {
                fullCmd += "                using " + Context.TypeManager.Now.FullName + ";";
            }
            
            fullCmd +=
@"                using System;
                using System.IO;
                using System.Text;
                using System.Collections.Generic;
                using Orc.Shell.Core;

                namespace Orc.Shell.Core.Dynamic 
                { 
                    public class DynamicClass
                    {
                        public Orc.Shell.Core.Context Context;

                        public void Save(String name, Object obj)
                        {
                            Context.Save(name,obj);
                        }

                        public Object My(String name)
                        {
                            return Context[name];
                        }

                        public void MethodInstance(Context context)
                        {
                            Context = context;
                            " + InputCmdString + @";
                        }
                    }
                }";

            CompilerResults cr = Context.CodeProvider.CompileAssemblyFromSource(Context.CompilerParameters, fullCmd);

            if (Context.Debug)
            {
                Console.WriteLine("Source:");
                Console.WriteLine("--------------------------------");
                Console.WriteLine(fullCmd);
                Console.WriteLine("--------------------------------");
                Console.WriteLine("Results");
            }

            if (cr.Errors.HasErrors)
            {
                Console.WriteLine("编译错误：");
                foreach (CompilerError err in cr.Errors)
                {
                    if (Context.Debug)
                    {
                        Console.WriteLine(String.Format("line {0}: {1}", err.Line, err.ErrorText));
                    }
                    else
                    {
                        Console.WriteLine(err.ErrorText);
                    }
                }
            }
            else
            {
                Assembly assem = cr.CompiledAssembly;
                Object dynamicObject = assem.CreateInstance("Orc.Shell.Core.Dynamic.DynamicClass");
                Type t = assem.GetType("Orc.Shell.Core.Dynamic.DynamicClass");
                MethodInfo minfo = t.GetMethod("MethodInstance");
                minfo.Invoke(dynamicObject, new Object[] { Context });
            }
        }
    }
}
